/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.specific;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * this extends the File class by providing methods to get relative paths to
 * this file, with respect to the location of another file. It also overrrides
 * the equals method to make it compare file names and not paths : foo and
 * bar/../foo are now equal (for unix)
 * 
 * @author Sebastien Ponce
 */
public class RelativeFile extends File {

    /**
     * Default constructor
     * 
     * @param parent the parent abstract pathname
     * @param child the child pathname string
     */
    public RelativeFile(File parent, String child) {
        super(parent, child);
    }

    /**
     * Default constructor
     * 
     * @param pathname a pathname string
     */
    public RelativeFile(String pathname) {
        super(pathname);
    }

    /**
     * This methods returns the shorter relative name of this File with respect
     * to the location of the File file.
     * 
     * @param file the reference File
     * @return the relative path from file to this File
     */
    public String getRelativePath(File file) {
        StringTokenizer fileStrTok, thisStrTok;
        try {
            fileStrTok = new StringTokenizer(file.getParentFile()
                    .getCanonicalPath(), File.separator);
            thisStrTok = new StringTokenizer(this.getCanonicalPath(),
                    File.separator);
        } catch (IOException e) {
            // let's use the absolute path instead of the canonical one
            fileStrTok = new StringTokenizer(file.getParentFile()
                    .getAbsolutePath(), File.separator);
            thisStrTok = new StringTokenizer(this.getAbsolutePath(),
                    File.separator);
        }
        String result = "";
        boolean equal = true;
        while (fileStrTok.hasMoreTokens() && thisStrTok.hasMoreTokens()
                && equal) {
            String fileTok = fileStrTok.nextToken();
            String thisTok = thisStrTok.nextToken();
            if (!fileTok.equals(thisTok)) {
                equal = false;
                for (int i = 0; i <= fileStrTok.countTokens(); i++) {
                    result = result + ".." + File.separator;
                }
                result = result + thisTok;
                if (thisStrTok.hasMoreTokens()) {
                    result = result + File.separator;
                }
            }
        }
        while (thisStrTok.hasMoreTokens()) {
            result = result + thisStrTok.nextToken();
            if (thisStrTok.hasMoreTokens()) {
                result = result + File.separator;
            }
        }
        return result;
    }

    /**
     * Tests this abstract pathname for equality with the given object. Returns
     * true if and only if the argument is not null and is an abstract pathname
     * that denotes the same file or directory as this abstract pathname. Note
     * that foo and bar/../foo are thus equal (under unix).
     * 
     * @param obj the object to be compared to this pathname
     */
    public boolean equals(Object obj) {
        if (obj instanceof File) {
            try {
                return this.getCanonicalFile().equals(
                        ((File) obj).getCanonicalFile());
            } catch (IOException e) {
            }
        }
        return false;
    }

}