/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import java.util.Enumeration;
import java.util.Vector;

import org.xml.sax.SAXParseException;

import ch.cern.lhcb.xmleditor.gui.MessageDisplayer;

/**
 * This class implements an error handler that stores all errors during parses
 * and can give them back
 * 
 * @author Sebastien Ponce
 */
public class ErrorHandler implements org.xml.sax.ErrorHandler {

    /**
     * This Vector stores all the errors occured during parsing
     */
    private Vector errorNodes = new Vector();

    /**
     * called when a warning occurs during parsing
     */
    public void warning(SAXParseException ex) {
        store(ex, "[Warning]");
    }

    /**
     * called when an error occurs during parsing
     */
    public void error(SAXParseException ex) {
        store(ex, "[Error]");
    }

    /**
     * called when a fatasl error occurs during parsing
     */
    public void fatalError(SAXParseException ex) {
        store(ex, "[Fatal Error]");
    }

    /**
     * displays all errors stored in this errorHandler
     * 
     * @param msgDis the MessageDisplayer used for error displaying
     */
    public void displayErrors(MessageDisplayer msgDis) {
        if (errorNodes.size() > 100) {
            msgDis
                    .displayMessage("*************************************************"
                            + "***************************\n* There were more than"
                            + " 100 errors while parsing. This is probably not xml. *\n"
                            + "*****************************************************"
                            + "***********************\n");
        } else {
            Enumeration errors = errorNodes.elements();
            while (errors.hasMoreElements()) {
                // appends errors to messageText
                msgDis.displayMessage((String) errors.nextElement() + "\n");
            }
            msgDis.displayMessage("\n");
        }
    }

    /**
     * resets the error handler by clearing all errors
     */
    public void reset() {
        errorNodes.clear();
    }

    /**
     * stores an error
     * 
     * @param ex the exception received due to this error
     * @param type a string giving the level of error (warning, error or fatal
     *        error)
     */
    private void store(SAXParseException ex, String type) {
        // build error text
        String errorString = type + " at line number, " + ex.getLineNumber()
                + ": " + ex.getMessage();
        // adds it to the error list
        errorNodes.add(errorString);
    }

    /**
     * says whether the last parsing did generate errors or not
     * 
     * @return true if it did
     */
    public boolean errorsExisting() {
        return !errorNodes.isEmpty();
    }

}