/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.net.URL;

import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import com.simulogic.cese.actions.AboutAction;
import com.simulogic.cese.actions.ExitAction;
import com.simulogic.cese.actions.LockRecAction;
import com.simulogic.cese.actions.ModManagerAction;
import com.simulogic.cese.actions.NewParamsAction;
import com.simulogic.cese.actions.NextHistoryAction;
import com.simulogic.cese.actions.NextRecAction;
import com.simulogic.cese.actions.OpenParamsAction;
import com.simulogic.cese.actions.OpenRecAction;
import com.simulogic.cese.actions.PrevHistoryAction;
import com.simulogic.cese.actions.PrevRecAction;
import com.simulogic.cese.actions.RecEditAction;
import com.simulogic.cese.actions.RecTestAction;
import com.simulogic.cese.actions.ReportBugAction;
import com.simulogic.cese.actions.SaveParamsAction;
import com.simulogic.cese.actions.SaveParamsAsAction;
import com.simulogic.cese.actions.SimOptionsAction;
import com.simulogic.cese.actions.StartSimAction;
import com.simulogic.cese.actions.StopSimAction;
import com.simulogic.cese.actions.UpdateAction;
import com.simulogic.cese.updates.UpdateFetcher;
import com.simulogic.cese.updates.UpdateOptions;
import com.simulogic.dataview.DataDesktop;
import com.simulogic.dataview.actions.AppendCursorsAction;
import com.simulogic.dataview.actions.AvrgAction;
import com.simulogic.dataview.actions.BuildIVAction;
import com.simulogic.dataview.actions.CascadeAction;
import com.simulogic.dataview.actions.CloseAction;
import com.simulogic.dataview.actions.CloseAllAction;
import com.simulogic.dataview.actions.CreateFrameAction;
import com.simulogic.dataview.actions.ExportResAction;
import com.simulogic.dataview.actions.PrintPrevAction;
import com.simulogic.dataview.actions.PrintResAction;
import com.simulogic.dataview.actions.RenameAction;
import com.simulogic.dataview.actions.ReplaceCursorsAction;
import com.simulogic.dataview.actions.SelectSweepAction;
import com.simulogic.dataview.actions.StatsAction;
import com.simulogic.dataview.actions.TileAction;
import com.simulogic.dataview.actions.TileHorAction;
import com.simulogic.dataview.actions.TileVerAction;

/**
 * Main CESE module
 */
public class CESE extends javax.swing.JFrame {

	private static CESE environment = null;

	private CreateFrameAction createFrameAction;

	private LockRecAction lockRecAction;

	private ModelManager manager;

	private DataDesktop desktop;

	private JMenu fileMenu;

	private JMenu modelMenu;

	private JMenu recordMenu;

	private JMenu analysisMenu;

	private JMenu windowMenu;

	private JMenu helpMenu;

	// Defaults for Main Help
	static final String helpsetName = "CESEHelp";

	// Main HelpSet & Broker
	HelpSet mainHS = null;

	HelpBroker mainHB;

	/**
	 * Returns help broker
	 * 
	 * @return help broker
	 */
	public HelpBroker getHelpBroker() {
		return mainHB;
	}

	/**
	 * Returns model manager
	 * 
	 * @return model manager
	 */
	public ModelManager getModelManager() {
		return manager;
	}

	/**
	 * Returns desktop
	 * 
	 * @return desktop
	 */
	public DataDesktop getDesktop() {
		return desktop;
	}

	/**
	 * Returns the environment
	 * 
	 * @return environment
	 */
	public static CESE getEnvironment() {
		return environment;
	}

	private void initComponents() {
		buildHelpSystem();

		desktop = new DataDesktop();
		CSH.setHelpIDString(desktop, "Visual.Desktop");

		manager = new ModelManager();

		setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);

		setIconImage(new ImageIcon(this.getClass().getResource(
				ResourceManager.getResource("icon_image"))).getImage());

		Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
		s.setSize(s.width - 50, s.height - 120);
		setTitle(ResourceManager.getResource("Title"));
		addWindowListener(new java.awt.event.WindowAdapter() {

			public void windowClosing(java.awt.event.WindowEvent evt) {
				exitEnvironment();
			}
		});

		getContentPane().setLayout(new java.awt.BorderLayout());
		setJMenuBar(buildMenu());
		Box toolBarPanel = Box.createHorizontalBox();
		toolBarPanel.add(buildModelToolbar());
		toolBarPanel.add(Box.createHorizontalGlue());
		toolBarPanel.add(buildRecordToolbar());
		getContentPane().add(toolBarPanel, "North");

		JSplitPane vertSplitPane = new javax.swing.JSplitPane();
		vertSplitPane.setOrientation(javax.swing.SwingConstants.VERTICAL);
		vertSplitPane.setContinuousLayout(false);
		vertSplitPane.setOneTouchExpandable(true);
		vertSplitPane.setRightComponent(manager);
		getContentPane().add(vertSplitPane, "Center");
		desktop.setPreferredSize(new java.awt.Dimension(s.width * 2 / 3,
				s.height));
		manager.setPreferredSize(new java.awt.Dimension(s.width / 3, s.height));
		vertSplitPane.setLeftComponent(desktop);
		vertSplitPane.setDividerLocation(0.9D);

		manager.setModelActions(false);
		manager.setRecordActions(false);
		manager.setHistoryActions(false);
		desktop.setWindowActions(false);
	}

	private void buildHelpSystem() {
		try {
			ClassLoader cl = CESE.class.getClassLoader();
			URL url = HelpSet.findHelpSet(cl, helpsetName);
			mainHS = new HelpSet(cl, url);
		} catch (Exception ee) {
			System.out.println("Help Set " + helpsetName + " not found");
			return;
		} catch (ExceptionInInitializerError ex) {
		}
		mainHB = mainHS.createHelpBroker();
		mainHB.enableHelpKey(getRootPane(), "top", null);
	}

	private JMenuBar buildMenu() {
		JMenuItem item;

		JMenuBar mainMenu;

		mainMenu = new javax.swing.JMenuBar();

		fileMenu = new javax.swing.JMenu();
		getFileMenu().setText(ResourceManager.getResource("File"));
		getFileMenu().setMnemonic('F');

		item = getFileMenu().add(new NewParamsAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('N', 2));
		CSH.setHelpIDString(item, "Commands.New_Parameters");
		item = getFileMenu().add(new OpenParamsAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('O', 2));
		CSH.setHelpIDString(item, "Commands.Open_Parameters");
		item= manager.getRecentParameters();
		item.setText(ResourceManager.getResource("Open_Recent_Parameters"));
		item.setMnemonic('R');
		getFileMenu().add(item);
		item = getFileMenu().add(new SaveParamsAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('S', 2));
		CSH.setHelpIDString(item, "Commands.Save_Parameters");
		item = getFileMenu().add(new SaveParamsAsAction(manager));
		CSH.setHelpIDString(item, "Commands.Save_Parameters_As");
		getFileMenu().addSeparator();

		item = getFileMenu().add(new ExportResAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('E', 2));
		CSH.setHelpIDString(item, "Commands.Export_Results");
		item = getFileMenu().add(new PrintResAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('P', 2));
		CSH.setHelpIDString(item, "Commands.Print_Results");
		item = getFileMenu().add(new PrintPrevAction(desktop));
		item.setMnemonic('P');
		CSH.setHelpIDString(item, "Commands.Print_Preview");
		getFileMenu().addSeparator();

		item = getFileMenu().add(new ExitAction());
		item.setAccelerator(KeyStroke.getKeyStroke('X', 8));
		CSH.setHelpIDString(item, "Commands.Exit");
		mainMenu.add(getFileMenu());

		modelMenu = new javax.swing.JMenu();
		getModelMenu().setText(ResourceManager.getResource("Model"));
		getModelMenu().setMnemonic('M');

		item = getModelMenu().add(new StartSimAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		CSH.setHelpIDString(item, "Commands.Start_Simulation");
		item = getModelMenu().add(new StopSimAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 2));
		CSH.setHelpIDString(item, "Commands.Stop_Simulation");
		getModelMenu().addSeparator();

		item = getModelMenu().add(new PrevHistoryAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 8));
		CSH.setHelpIDString(item, "Commands.Previous_History");
		item = getModelMenu().add(new NextHistoryAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 8));
		CSH.setHelpIDString(item, "Commands.Next_History");
		getModelMenu().addSeparator();

		item = getModelMenu().add(new ModManagerAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('I', 2));
		CSH.setHelpIDString(item, "Commands.Model_Manager");
		item = getModelMenu().add(new SimOptionsAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('M', 2));
		CSH.setHelpIDString(item, "Commands.Simulation_Options");
		getModelMenu().addSeparator();

		createFrameAction = new CreateFrameAction(desktop);
		item = getModelMenu().add(createFrameAction.getItem());
		item.setAccelerator(KeyStroke.getKeyStroke('W', 2));
		CSH.setHelpIDString(item, "Commands.Frame_Select");
		getModelMenu().add(manager.getExperimentTypeChanger());
		mainMenu.add(getModelMenu());

		recordMenu = new javax.swing.JMenu();
		getRecordMenu().setText(ResourceManager.getResource("Record"));
		getRecordMenu().setMnemonic('R');

		item = getRecordMenu().add(new OpenRecAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('R', 2));
		CSH.setHelpIDString(item, "Commands.Open_Record");
		item= manager.getRecordBuilder().getRecentRecords();
		item.setText(ResourceManager.getResource("Open_Recent_Records"));
		item.setMnemonic('E');
		getRecordMenu().add(item);
		item = getRecordMenu().add(new RecEditAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('D', 2));
		CSH.setHelpIDString(item, "Commands.Record_Editor");
		item = getRecordMenu().add(new RecTestAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke('T', 2));
		CSH.setHelpIDString(item, "Commands.Test_Record");
		getRecordMenu().addSeparator();

		item = getRecordMenu().add(new PrevRecAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 2));
		CSH.setHelpIDString(item, "Commands.Previous_Record");
		item = getRecordMenu().add(new NextRecAction(manager));
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 2));
		CSH.setHelpIDString(item, "Commands.Next_Record");
		getRecordMenu().addSeparator();

		lockRecAction = new LockRecAction(manager);
		item = getRecordMenu().add(lockRecAction.getItem());
		item.setAccelerator(KeyStroke.getKeyStroke('L', 2));
		CSH.setHelpIDString(item, "Commands.Lock_Record");
		mainMenu.add(getRecordMenu());

		analysisMenu = new javax.swing.JMenu();
		getAnalysisMenu().setText(ResourceManager.getResource("Analysis"));
		getAnalysisMenu().setMnemonic('A');

		item = getAnalysisMenu().add(new BuildIVAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('U', 2));
		CSH.setHelpIDString(item, "Commands.Build_IV");
		item = getAnalysisMenu().add(new StatsAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('B', 2));
		CSH.setHelpIDString(item, "Commands.Basic_Statistics");
		item = getAnalysisMenu().add(new AvrgAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('G', 2));
		CSH.setHelpIDString(item, "Commands.Channel_Average");
		getAnalysisMenu().addSeparator();
		
		item = getAnalysisMenu().add(new AppendCursorsAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('A', 3));
		CSH.setHelpIDString(item, "Commands.Append_Cursors");
		item = getAnalysisMenu().add(new ReplaceCursorsAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('R', 3));
		CSH.setHelpIDString(item, "Commands.Replace_Cursors");
		getAnalysisMenu().addSeparator();

		item = getAnalysisMenu().add(new SelectSweepAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('S', 8));
		CSH.setHelpIDString(item, "Commands.Select_Sweeps");
		mainMenu.add(getAnalysisMenu());

		windowMenu = new javax.swing.JMenu();
		getWindowMenu().setText(ResourceManager.getResource("Window"));
		getWindowMenu().setMnemonic('W');

		item = getWindowMenu().add(new TileHorAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('Z', 8));
		CSH.setHelpIDString(item, "Commands.Tile_Horizontal");
		item = getWindowMenu().add(new TileVerAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('V', 8));
		CSH.setHelpIDString(item, "Commands.Tile_Vertical");
		item = getWindowMenu().add(new TileAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('T', 8));
		CSH.setHelpIDString(item, "Commands.Tile");
		item = getWindowMenu().add(new CascadeAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('C', 8));
		CSH.setHelpIDString(item, "Commands.Cascade");
		getWindowMenu().addSeparator();

		item = getWindowMenu().add(new CloseAllAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('O', 8));
		CSH.setHelpIDString(item, "Commands.Close_All");
		item = getWindowMenu().add(new CloseAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('L', 8));
		CSH.setHelpIDString(item, "Commands.Close");
		getWindowMenu().addSeparator();

		item = getWindowMenu().add(new RenameAction(desktop));
		item.setAccelerator(KeyStroke.getKeyStroke('N', 8));
		CSH.setHelpIDString(item, "Commands.Rename");

		getWindowMenu().add(desktop.getWindowMenu());
		mainMenu.add(getWindowMenu());

		helpMenu = new javax.swing.JMenu();
		getHelpMenu().setText(ResourceManager.getResource("Help"));
		getHelpMenu().setMnemonic('H');

		if (mainHB != null) {
			item = getHelpMenu().add(
					new JMenuItem(ResourceManager.getResource("Contents")));
			item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
			item.addActionListener(new CSH.DisplayHelpFromFocus(mainHB));
			item = getHelpMenu().add(
					new JMenuItem(ResourceManager.getResource("What_this")));
			item.setIcon(new ImageIcon(this.getClass().getResource(
					ResourceManager.getResource("context_image"))));
			item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 1));
			item.addActionListener(new CSH.DisplayHelpAfterTracking(mainHB));
			getHelpMenu().addSeparator();
		}

		item = getHelpMenu().add(new ReportBugAction());
		CSH.setHelpIDString(item, "Commands.Report_Bug");
		item = getHelpMenu().add(new UpdateAction());
		CSH.setHelpIDString(item, "Commands.Check_for_Updates");
		getHelpMenu().addSeparator();

		item = getHelpMenu().add(new AboutAction());
		item.setMnemonic('A');
		CSH.setHelpIDString(item, "Commands.About");

		mainMenu.add(getHelpMenu());

		return mainMenu;
	}

	private JToolBar buildModelToolbar() {
		javax.swing.AbstractButton b;

		JToolBar modelToolBar = new javax.swing.JToolBar();

		modelToolBar.setLayout(new javax.swing.BoxLayout(modelToolBar, 0));
		b = alignToolButton(modelToolBar.add(new NewParamsAction(manager)));
		CSH.setHelpIDString(b, "Commands.New_Parameters");
		b = alignToolButton(modelToolBar.add(new OpenParamsAction(manager)));
		CSH.setHelpIDString(b, "Commands.Open_Parameters");
		b = alignToolButton(modelToolBar.add(new SaveParamsAction(manager)));
		CSH.setHelpIDString(b, "Commands.Save_Parameters");
		modelToolBar.addSeparator();
		b = alignToolButton(modelToolBar.add(new PrintResAction(desktop)));
		CSH.setHelpIDString(b, "Commands.Print_Results");
		modelToolBar.addSeparator();
		b = alignToolButton(modelToolBar.add(new ModManagerAction(manager)));
		CSH.setHelpIDString(b, "Commands.Model_Manager");
		modelToolBar.addSeparator();

		return modelToolBar;
	}

	private JToolBar buildRecordToolbar() {
		javax.swing.AbstractButton b;

		JToggleButton lockrec;
		JTextField field;

		JToolBar recordToolBar = new javax.swing.JToolBar();

		recordToolBar.setLayout(new javax.swing.BoxLayout(recordToolBar, 0));
		b = alignToolButton(recordToolBar.add(new OpenRecAction(manager)));
		CSH.setHelpIDString(b, "Commands.Open_Record");
		recordToolBar.addSeparator();
		b = alignToolButton(recordToolBar.add(new PrevRecAction(manager)));
		CSH.setHelpIDString(b, "Commands.Previous_Record");
		lockrec = lockRecAction.getButton();
		b = alignToolButton(lockrec);
		CSH.setHelpIDString(b, "Commands.Lock_Record");
		field = manager.getRecordBuilder().getRecNumberField();
		field.setMaximumSize(lockrec.getPreferredSize());
		CSH.setHelpIDString(field, "Commands.Record_Number");
		recordToolBar.add(field);
		b = alignToolButton(recordToolBar.add(new NextRecAction(manager)));
		CSH.setHelpIDString(b, "Commands.Next_Record");
		recordToolBar.add(lockrec);
		recordToolBar.addSeparator();

		b = alignToolButton(recordToolBar.add(new StartSimAction(manager)));
		CSH.setHelpIDString(b, "Commands.Start_Simulation");
		b = alignToolButton(recordToolBar.add(new StopSimAction(manager)));
		CSH.setHelpIDString(b, "Commands.Stop_Simulation");
		recordToolBar.addSeparator();
		b = alignToolButton(recordToolBar.add(new PrevHistoryAction(manager)));
		CSH.setHelpIDString(b, "Commands.Previous_History");
		b = alignToolButton(recordToolBar.add(new NextHistoryAction(manager)));
		CSH.setHelpIDString(b, "Commands.Next_History");
		recordToolBar.addSeparator();

		lockrec = createFrameAction.getButton();
		b = alignToolButton(lockrec);
		CSH.setHelpIDString(b, "Commands.Frame_Select");
		recordToolBar.add(lockrec);
		recordToolBar.addSeparator();

		return recordToolBar;
	}

	private javax.swing.AbstractButton alignToolButton(
			javax.swing.AbstractButton b) {
		b.setText(null);
		b.setMargin(new Insets(0, 0, 0, 0));
		return b;
	}

	protected void updateCheck() {
		new IntroWindow(environment);
			com.simulogic.utils.SwingWorker worker = new com.simulogic.utils.SwingWorker() {
			boolean update = false;

			public Object construct() {
				try {
					UpdateFetcher fetcher = new UpdateFetcher();
					if (fetcher.isUpdateTime()) {
						fetcher.setUpdateTime();

						if (fetcher.isnewVersionAvailable())
							update = true;
					}
				} catch (Exception e) {
				}
				return ResourceManager.getResource("Done");
			}

			public void finished() {
				if (update) {
					UpdateOptions options = new UpdateOptions(CESE
							.getEnvironment());
					options.setVisible(true);
					options.toFront();
				}
			}
		};

		worker.start();
	}
	
	/**
	 * Exits environment
	 */
	public void exitEnvironment() {
		int n = JOptionPane.showConfirmDialog(CESE.getEnvironment(),
				ResourceManager.getResource("Exit_program?"), ResourceManager
						.getResource("Exit"), JOptionPane.YES_NO_OPTION);
		if (n == JOptionPane.NO_OPTION)
			return;

		setVisible(false);
		System.exit(0);
	}

	/**
	 * Main CESE start method
	 * 
	 * @param args
	 *            command line arguments
	 */
	public static void main(java.lang.String[] args) {
		SplashWindow splash = new SplashWindow(null);
		
		environment = new CESE();
		environment.initComponents();

		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				environment.pack();
				environment.setVisible(true);
				environment.getModelManager().getHome().populateUserHome();
			}
		});

		splash.closeSplash();

		environment.updateCheck();
	}

	/**
	 * @return Returns the fileMenu.
	 */
	private JMenu getFileMenu() {
		return fileMenu;
	}

	/**
	 * @return Returns the modelMenu.
	 */
	private JMenu getModelMenu() {
		return modelMenu;
	}

	/**
	 * @return Returns the recordMenu.
	 */
	private JMenu getRecordMenu() {
		return recordMenu;
	}

	/**
	 * @return Returns the analysisMenu.
	 */
	private JMenu getAnalysisMenu() {
		return analysisMenu;
	}

	/**
	 * @return Returns the windowMenu.
	 */
	private JMenu getWindowMenu() {
		return windowMenu;
	}

	/**
	 * @return Returns the helpMenu.
	 */
	private JMenu getHelpMenu() {
		return helpMenu;
	}
}