/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;

import ch.cern.lhcb.xmleditor.specific.XmlFile;
import ch.cern.lhcb.xmleditor.specific.XmlFilesManager;
import ch.cern.lhcb.xmleditor.undo.CompoundEdit;
import ch.cern.lhcb.xmleditor.xerces.ElementImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this is a listener to a popup menu appeared on an XmlTreeNode
 * 
 * @author Sebastien Ponce
 */
public class XmlPopupListener implements ActionListener {

    /**
     * the TreeNode on which the popup menu we are listening to appeared
     */
    private XmlTreeNode treeNode;

    /**
     * constructor
     * 
     * @param treeNode TreeNode on which the popup menu we will listen to
     *        appeared
     */
    public XmlPopupListener(XmlTreeNode treeNode) {
        this.treeNode = treeNode;
    }

    /**
     * called each time an action occurs
     * 
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        // gets the XmlTree displaying this node
        XmlTree jTree = treeNode.getXmlTree();
        // gets the source
        JMenuItem source = (JMenuItem) e.getSource();
        // if the edit reference item was hit
        if (source.getText() == XmlMouseListener.EDIT_REFERENCE) {
            // begins a compoundedit for undo management since this action
            // splits in two parts : derreferencing and editing the node.
            // this compound edit is non ended here but after end of edition
            jTree.getTreeView().getEditor().getUndoManager().addEdit(
                    new CompoundEdit(jTree.getTreeView().getEditor()));
            // gets the oldNode
            XmlTreeNode oldNode = treeNode.getOldNode();
            // removes the subtree under the edited node and reput the
            // previous reference node
            jTree.undoableCollapseReference(treeNode);
            // edits the previous node
            TreePath path = new TreePath(oldNode.getPath());
            jTree.getUI().startEditingAtPath(jTree, path);
        } else if (source.getText() == XmlMouseListener.CLOSE_REFERENCE) {
            // removes the subtree under the edited node and reput the
            // previous reference node
            jTree.undoableCollapseReference(treeNode);
        } else if (source.getText() == XmlMouseListener.REMOVE_ELEMENT) {
            // gets the node to be removed
            TreeElement node = treeNode.getNode();
            // gets the parent
            ElementImpl parent = (ElementImpl) node.getParentNode();
            // removes the node from the document
            parent.undoableRemoveChild(node);
        } else if (source.getText() == XmlMouseListener.REMOVE_DEREF_ELEMENT) {
            // begins a compoundedit for undo management since this action
            // splits in two parts : derreferencing and removing the node
            CompoundEdit edit = new CompoundEdit(jTree.getTreeView()
                    .getEditor());
            jTree.getTreeView().getEditor().getUndoManager().addEdit(edit);
            // removes the subtree under the edited node and reput the
            // previous reference node
            jTree.undoableCollapseReference(treeNode);
            // gets the reference node
            TreeElement node = treeNode.getNode();
            // gets the parent
            ElementImpl parent = (ElementImpl) node.getParentNode();
            // removes the node from the document
            parent.undoableRemoveChild(node);
            // closes the compound edit
            edit.end();
        } else if (source.getText() == XmlMouseListener.SAVE_ITEM) {
            // gets the file containing to the clicked node
            XmlFile file = XmlFilesManager.getFileFromNode(treeNode.getNode());
            // saves the file if necessary (and name it before if needed)
            if (file.hasName) {
                if (!file.upToDate || (file.lastModified < file.lastModified())) {
                    file.save();
                }
            } else {
                // names and saves the file
                jTree.getTreeView().nameAndSaveFile(file);
            }
        } else if (source.getText() == XmlMouseListener.VIEW_SOURCE_ITEM) {
            // gets the corresponding file
            XmlFile file = XmlFilesManager.getFileFromNode(treeNode.getNode());
            // opens a new TextView
            jTree.getTreeView().getEditor().createNewTextView(file);
        } else {
            // gets the menu that was invoked
            JMenu menu = (JMenu) ((JPopupMenu) source.getParent()).getInvoker();
            // gets the node concerned
            ElementImpl node = (ElementImpl) treeNode.getNode();
            // if it was "create new node", creates a new node
            if (menu.getText() == XmlMouseListener.CREATE_ELEMENT) {
                // the name of the new node
                String nodeName = source.getText();
                // depending on the node name, creates it
                if (nodeName.equals(XmlMouseListener.COMMENT)) {
                     node.createComment(jTree, treeNode, jTree
                            .getTreeView(), true);
                } else if (nodeName.equals(XmlMouseListener.VALUE)) {
                    node.createValue(jTree, treeNode, jTree
                            .getTreeView(), true);
                } else {
                    node.createNode(nodeName, treeNode, jTree
                            .getTreeView(), true);
                }
            } else if (menu.getText() == XmlMouseListener.CREATE_ATTRIBUTE) {
                // else if it was "create new attribute", so creates an
                // attribute
                node.createAttribute(source.getText(), jTree
                        .getTreeView(), true);
            } else if (menu.getText() == XmlMouseListener.REMOVE_ATTRIBUTE) {
                // else if it was "remove attribute", removes the selected
                // attribute
                node.removeAttribute(source.getText(), jTree
                        .getTreeView());
            }
        }
    }

}