/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;

import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.specific.Model;
import ch.cern.lhcb.xmleditor.specific.XmlDisplay;
import ch.cern.lhcb.xmleditor.specific.XmlFile;
import ch.cern.lhcb.xmleditor.specific.XmlFilesManager;
import ch.cern.lhcb.xmleditor.transfer.DragGestureRecognizer;
import ch.cern.lhcb.xmleditor.transfer.DropTargetListener;
import ch.cern.lhcb.xmleditor.undo.UndoableCloseReference;
import ch.cern.lhcb.xmleditor.undo.UndoableOpenReference;
import ch.cern.lhcb.xmleditor.xerces.DocumentImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this is an extension of JTree for displaying multiple files Xml trees. This
 * tree knows every file it is using and is able to save itself.
 * 
 * @author Sebastien Ponce
 */
public class XmlTree extends JTree {

    /**
     * true if a drop copy action is in work. False if it is a move or link
     * action
     */
    // the use of the variable copyOrLinkActionInProgress is only a workaround
    // to a bug
    // found in the sun jvm on windows. This should be removed as soon as this
    // bug get fixed -- FIXME
    public boolean copyOrLinkActionInProgress;

    /**
     * an error message displayer
     */
    private MessageDisplayer msgDis;

    /**
     * a pointer on the TreeView that uses this Tree
     */
    private TreeView treeView;

    /**
     * the currently dragged row if any. -1 if none
     */
    private int draggedRow;

    /**
     * says whether the drag action is really on the drag row or on one of the
     * borders of the row (either top or bottom border). This should only take
     * the values javax.swing.SwingConstants.TOP,
     * javax.swing.SwingConstants.CENTER or javax.swing.SwingConstants.BOTTOM
     */
    private int draggedRowOffset;

    /**
     * constructor
     * 
     * @param msgDis a MessageDisplayer for error displaying
     * @param treeView the TreeView in which this tree is displayed
     */
    public XmlTree(MessageDisplayer msgDis, TreeView treeView) {
        // calls super constructor and creates the Model
        super(new Model());

        // initialize the fields
        DragSource dragSource = DragSource.getDefaultDragSource();
        this.msgDis = msgDis;
        this.treeView = treeView;
        this.draggedRow = -1;
        this.draggedRowOffset = javax.swing.SwingConstants.CENTER;

        // let the model know its owner
        this.getXmlModel().setXmlTree(this);

        // initiates the drag and drop capabilities
        dragSource.createDefaultDragGestureRecognizer(this,
                DnDConstants.ACTION_COPY_OR_MOVE | DnDConstants.ACTION_LINK,
                new DragGestureRecognizer(this, dragSource));
        DropTargetListener dropTargetListener = new DropTargetListener(this,
                treeView);
        new DropTarget(this, dropTargetListener);

        // sets some properties
        this.setRootVisible(false);
        TreeCellRenderer treeCellRenderer = new TreeCellRenderer(treeView
                .getEditor().getIconsManager());
        this.setCellRenderer(treeCellRenderer);
        this.setCellEditor(new TreeCellEditor(this, msgDis));
        this.setInvokesStopCellEditing(true);
        this.setEditable(true);
        this.putClientProperty("JTree.lineStyle", "Angled");
        this.getSelectionModel().setSelectionMode(
                TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        this.addTreeWillExpandListener(new TreeWillExpandListener(this));
        this.addMouseListener(new XmlMouseListener(this));
        this.addKeyListener(new XmlKeyListener(this));
    }

    /////////////////////////////////////////
    ////// ACCESSORS //////
    /////////////////////////////////////////

    /**
     * gets the model associated to this tree
     * 
     * @return the Model
     */
    public Model getXmlModel() {
        return (Model) super.getModel();
    }

    /**
     * gets the TreeView displaying this XmlTree
     * 
     * @return the TreeView displaying this XmlTree
     */
    public TreeView getTreeView() {
        return treeView;
    }

    ////////////////////////////////////////////
    ////// NODE RELATED METHODS //////
    ////////////////////////////////////////////

    /**
     * gets the XmlDisplay corresponding to a given node
     * 
     * @param node the node we want to know with which XmlDisplay it is
     *        associated
     * @return the XmlDisplay associated with node
     */
    public static XmlDisplay getXmlDisplayFromNode(Node node) {
        XmlFile file = XmlFilesManager.getFileFromNode(node);
        if (file != null) {
            return file.getXmlDisplay();
        } else {
            return null;
        }
    }

    /**
     * closes a reference by replacing a treeNode by the one that used to be
     * displayed at the same place before the reference was opened.
     * 
     * @param treeNode the treeNode to replace
     */
    public void collapseReference(XmlTreeNode treeNode) {
        // does nothing if there's no previous node (should never happend)
        if (treeNode.getOldNode() == null) { return; }
        // retrieves the parent of this node
        XmlTreeNode parent = treeNode.getParentTreeNode();
        // retrieves the index of treeNode inside its parent
        int index = parent.getIndex(treeNode);
        // if the node was selected, remembers it
        boolean selected = this
                .isPathSelected(new TreePath(treeNode.getPath()));
        // removes treeNode from the tree
        Model model = this.getXmlModel();
        model.removeNodeFromParent(treeNode);
        // updates the representations of the underlying node
        treeNode.getNode().removeTreeNode(treeNode);
        // puts the previous node in the tree
        model.insertNodeInto(treeNode.getOldNode(), parent, index);
        // update the list of files used in this tree
        XmlFile file = ((DocumentImpl) treeNode.getNode()
                .getOwnerDocument()).getXmlFile();
        treeView.removeRefOnFile(file);
        // selects it if necessary
        if (selected) {
            this
                    .addSelectionPath(new TreePath(treeNode.getOldNode()
                            .getPath()));
            // update the attribute display
            treeView.getEditorPane().displayNode(
                    treeNode.getOldNode().getNode());
        }
        // removes the pointers between the two nodes
        treeNode.getOldNode().setNewNode(null);
        treeNode.setOldNode(null);
    }

    /**
     * expands the XmlTreeNode given as argument if it is a reference on another
     * node. Returns true if the node was dereferenced.
     * 
     * @param treeNode the XmlTreeNode to expand
     * @return true if the node was dereferenced.
     */
    public boolean expandReference(XmlTreeNode treeNode) {
        // check whether it is a reference node
        if (treeNode.isReference()) {
            // if yes, replaces the current node by the real one
            TreeElement linkNode = treeView.resolveLink(treeNode.getNode());
            if (linkNode != null) {
                // note the position in the tree
                int row = this.getRowForPath(new TreePath(treeNode.getPath()));
                boolean selected = this.isRowSelected(row);
                // replaces the node
                this.getXmlModel().replaceNode(linkNode,
                        treeNode);
                // if the node that was replaced was selected,
                // then select the new one
                if (selected) {
                    this.addSelectionRow(row);
                }
                // returns
                return true;
            } else {
                msgDis
                        .popupMessage("An error occured while resolving the reference");
                msgDis
                        .displayMessage("The reference you tried to access must be invalid. "
                                + "Either the file does not exist or the target in the "
                                + "file doesn't exist.\n");
            }
        }
        return false;
    }

    /**
     * replaces a treeNode by the one that used to be displayed at the same
     * place before him. Makes the UndoManager aware of everything.
     * 
     * @param treeNode the treeNode to replace
     */
    public void undoableCollapseReference(XmlTreeNode treeNode) {
        XmlTreeNode oldNode = treeNode.getOldNode();
        this.collapseReference(treeNode);
        treeView.getEditor().getUndoManager().addEdit(
                new UndoableCloseReference(
                        oldNode.getNode().getOwnerDocument(), oldNode));
    }

    /**
     * expands the XmlTreeNode given as argument if it is a reference on another
     * node. If sucessfull, returns true. Makes the UndoManager aware of
     * everything.
     * 
     * @param treeNode the XmlTreeNode to expand
     * @return true if successfull
     */
    public boolean undoableExpandReference(XmlTreeNode treeNode) {
        Document ownerDocument = treeNode.getNode().getOwnerDocument();
        boolean success = this.expandReference(treeNode);
        if (success) {
            treeView.getEditor().getUndoManager().addEdit(
                    new UndoableOpenReference(ownerDocument, treeNode));
        }
        return success;
    }

    /**
     * sets the currently dragged row
     * 
     * @param row the currently dragged row
     */
    public void setDraggedRow(int row) {
        this.draggedRow = row;
    }

    /**
     * returns the currently dragged row
     * 
     * @return the currently dragged row
     */
    public int getDraggedRow() {
        return draggedRow;
    }

    /**
     * sets the currently dragged row offset
     * 
     * @param offset the currently dragged row offset
     */
    public void setDraggedRowOffset(int offset) {
        this.draggedRowOffset = offset;
    }

    /**
     * returns the currently dragged row offset
     * 
     * @return the currently dragged row offset
     */
    public int getDraggedRowOffset() {
        return draggedRowOffset;
    }

}