/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.specific;

import java.io.File;
import java.util.Enumeration;

import javax.swing.JFileChooser;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.simulogic.cese.UserHome;

import ch.cern.lhcb.xmleditor.Editor;
import ch.cern.lhcb.xmleditor.gui.MessageDisplayer;
import ch.cern.lhcb.xmleditor.xerces.DocumentImpl;

/**
 * This class is a kind of accessor for Xml documents. It provides two main
 * services : getting a document out of a fileName and resolving references
 * found in an Xml document. Appart from that, this class implements a cache on
 * Xml documents.
 * 
 * @author Sebastien Ponce
 */
public class XmlFilesManager {

    /**
     * The default name for a new file
     */
    private static final String BASE_FILE_NAME = "untitled";

    /**
     * the number of file created from scratch until now
     */
    private int fileCreatedNb;

    /**
     * this pool keeps every file opened and parsed together with the number of
     * places where it is used. Each XmlTree using this file should acts as only
     * one user whatever its usage of the file can be. Using a file means here
     * keep a reference on it, full stop.
     * 
     */
    private Pool refTable;

    /**
     * this pool keeps every file opened and parsed together with the number of
     * places where it is effectively used. Each XmlTree using this file should
     * acts as only one user whatever its usage of the file can be. Using a file
     * means here really work on it. Thus, a file should be removed from this
     * table while it is not modified and reput there when needed.
     * 
     */
    private Pool useTable;

    /**
     * the editor using this Files Manager
     */
    private Editor editor;

    /**
     * a file chooser to browse the file system
     */
    final private JFileChooser fileChooser;

    /**
     * Default constructor
     * 
     * @param editor the XmlEditor using this XmlFilesManager
     */
    public XmlFilesManager(Editor editor) {
        super();
        this.editor = editor;
        fileCreatedNb = 0;
        refTable = new Pool();
        useTable = new Pool();
        fileChooser = new JFileChooser(new java.io.File(UserHome.CESE_HOME
                + UserHome.FS + editor.getSetup().getDataLocation()));
    }

    /**
     * gets a DOM document from a file If this file was already accessed, just
     * reuse it, otherwise, open it, parse it and use it.
     * 
     * @param currentFile the current browsed file
     * @param fileName the name of the file to be browsed. Can be relative to
     *        the currentFile location
     * @param msgDis the MessageDisplayer to use for error displaying
     * @return the parsed file
     */
    public XmlFile getXmlFile(XmlFile currentFile, String fileName,
            MessageDisplayer msgDis) {
        // first parses the fileName
        XmlFile file = new XmlFile(currentFile, fileName, msgDis, editor);
        // then call a private routine that does the job
        return getAndParseDocument(file, msgDis);
    }

    /**
     * get a DOM document from a file If this file was already accessed, just
     * reuse it, otherwise, open it, parse it and use it.
     * 
     * @param fileName the name of the file to be browsed.
     * @param msgDis the MessageDisplayer to use for error displaying
     * @return the parsed file
     */
    public XmlFile getXmlFile(String fileName, MessageDisplayer msgDis) {
        // first parses the fileName
        XmlFile file = new XmlFile(fileName, msgDis, editor);
        // then call a private routine that does the job
        return getAndParseDocument(file, msgDis);
    }

    /**
     * tries to name an unnamed file by asking the user returns a file with the
     * new name or null if the operation was canceled
     * 
     * @param oldFile the old, unnamed file
     * @return the new, named file
     */
    public XmlFile nameNewFile(XmlFile oldFile) {
        // first check that the oldfile has no name
        if (oldFile.hasName) { return oldFile; }
        // asks the user for a name
        fileChooser.setDialogTitle("Save file as");
        int validFile = fileChooser.showSaveDialog(editor);
        // check if the file was approved or canceled
        if (validFile == JFileChooser.APPROVE_OPTION) {
            String fileName = fileChooser.getSelectedFile().getAbsolutePath();
            // creates a file with the new name
            XmlFile result = new XmlFile(oldFile, fileName, editor);
            // adds the new file to the list of parsed files
            refTable.addRef(result);
            useTable.addRef(result);
            // returns
            return result;
        }
        return null;
    }

    /**
     * gets a DOM document from a file If this file was already accessed, just
     * reuse it, otherwise, open it, parse it and use it.
     * 
     * @param file the file to be browsed
     * @param msgDis the MessageDisplayer used for error displaying
     * @return the parsed file
     */
    private XmlFile getAndParseDocument(XmlFile file, MessageDisplayer msgDis) {
        // true if the file we are looking for is already opened
        boolean alreadyOpened = false;
        // try to find the file in the list of already opened ones
        Enumeration enm = refTable.getPoolContent();
        while (enm.hasMoreElements()) {
            XmlFile openedFile = (XmlFile) enm.nextElement();
            if (file.equals(openedFile)) {
                file = openedFile;
                alreadyOpened = true;
                break;
            }
        }
        // if the file is not already opened, parse it and add it into the list
        if (!alreadyOpened) {
            // if the parsing was successfull
            if (file.parse()) {
                // tries to parse the dtd
                file.setXmlDisplay(editor.getXmlDisplaysManager()
                        .getXmlDisplay(file, msgDis));
                // adds it to the list of parsed files
            } else {
                // else return
                return null;
            }
        }
        // adds a reference on the new file
        refTable.addRef(file);
        useTable.addRef(file);
        // finally gets the document from the file
        return file;
    }

    /**
     * gets the XmlFile corresponding to a given node
     * 
     * @param node the node we want to know with which file it is associated
     * @return the XmlFile associated with node
     */
    public static XmlFile getFileFromNode(Node node) {
        Document document = node.getOwnerDocument();
        if (document != null) {
            return ((DocumentImpl) document).getXmlFile();
        } else {
            // maybe node was the document
            if (node instanceof DocumentImpl) {
                return ((DocumentImpl) node).getXmlFile();
            } else {
                return null;
            }
        }
    }

    /**
     * called when a file is closed. The file will only be closed if nobody else
     * uses it. The closing of a file only deals with the number of references
     * on this file, not on the number of objects using it. Thus, one should
     * call closeableFile before closeFile if he wants to close a used file.
     * 
     * @param file the file to close
     */
    public void closeFile(XmlFile file) {
        refTable.removeRef(file);
        if (refTable.getRefNumber(file) == 0) {
            // removes the file from the lists of open ones
            refTable.remove(file);
            useTable.remove(file);
            // closes the associated display
            editor.getXmlDisplaysManager().closeDisplay(file.getXmlDisplay());
        }
    }

    /**
     * called when a file is closable. This means it is not used anymore but
     * could be reused in the future. It is thus kept as referenced but not used
     * 
     * @param file the file to close
     */
    public void closableFile(XmlFile file) {
        useTable.removeRef(file);
        if (useTable.getRefNumber(file) == 0) {
            // removes the UndoableEdits corresponding to this file
            // in the UndoManager
            editor.getUndoManager().conditionnalDieAndTrim(file.getDocument());
        }
    }

    /**
     * creates a new file name and returns it. This name will be used the next
     * time getNewFile will be called
     */
    public String getNewFileName() {
        fileCreatedNb = fileCreatedNb + 1;
        return BASE_FILE_NAME + fileCreatedNb;
    }

    /**
     * creates a new xml file from scratch
     * 
     * @param docType the name of the document type
     * @param dtd the file containing the dtd
     * @param msgDis the MessageDisplayer to use for error displaying
     */
    public XmlFile getNewFile(String docType, File dtd, MessageDisplayer msgDis) {
        XmlFile file = new XmlFile(BASE_FILE_NAME + fileCreatedNb, docType, dtd
                .getAbsolutePath(), msgDis, editor);
        // tries to parse the dtd
        file.setXmlDisplay(editor.getXmlDisplaysManager().getXmlDisplay(file,
                msgDis));
        return file;
    }

    /**
     * returns a FileChooser The same fileChooser is return for successive
     * calls. This has the advantage to let it keep its status between two
     * usages (eg, the current directory)
     * 
     * @return a FileChooser
     */
    public JFileChooser getFileChooser() {
        return fileChooser;
    }

}